@@ -5,8 +5,9 @@
 #include <vector>
 #include <cmath>
 
-#include <dune/common/function.hh>
 #include <dune/common/bitsetvector.hh>
+#include <dune/common/indices.hh>
+#include <dune/common/transpose.hh>
 
 #include <dune/geometry/quadraturerules.hh>
 
@@ -63,41 +64,36 @@ void getLocalMatrix(const LocalView& localView,
     // Position of the current quadrature point in the reference element
     const auto quadPos = quadPoint.position();
 
-    // The transposed (inverse) Jacobian of the map from the reference element to the element
-    const auto& jacobianTransposed = geometry.jacobianTransposed(quadPos);
+    // The transposed inverse Jacobian of the map from the reference element to the element
+    const auto jacInvTrans = geometry.jacobianInverseTransposed(quadPos);
 
-    // The multiplicative factor in the integral transformation formula and its inverse
+    // The multiplicative factor in the integral transformation formula
     const auto integrationElement = geometry.integrationElement(quadPos);
-    const auto invIntegrationElement = 1.0 / integrationElement;
 
     ///////////////////////////////////////////////////////////////////////////
-    // Shape functions - flux - [assume H(div) elements]
+    // Shape functions - flux
     ///////////////////////////////////////////////////////////////////////////
 
-    // Values of the flux shape functions on the reference element
-    std::vector<FieldVector<double,dim> > fluxReferenceValues(fluxLocalFiniteElement.size());
-    fluxLocalFiniteElement.localBasis().evaluateFunction(quadPos, fluxReferenceValues);
-
-    // Values of the shape functions on the current element
+    // Values of the flux shape functions on the current element
     std::vector<FieldVector<double,dim> > fluxValues(fluxLocalFiniteElement.size());
-    for (size_t i=0; i<fluxValues.size(); i++)
-    {
-      jacobianTransposed.mtv(fluxReferenceValues[i], fluxValues[i]);
-      fluxValues[i] *= invIntegrationElement;
-    }
+    fluxLocalFiniteElement.localBasis().evaluateFunction(quadPos, fluxValues);
 
-    // Gradients of the flux shape functions on the reference element
-    std::vector<FieldMatrix<double,dim,dim> > fluxReferenceGradients(fluxLocalFiniteElement.size());
-    fluxLocalFiniteElement.localBasis().evaluateJacobian(quadPos, fluxReferenceGradients);
+    // Gradients of the flux shape function gradients on the reference element
+    std::vector<FieldMatrix<double,dim,dim> > fluxReferenceJacobians(fluxLocalFiniteElement.size());
+    fluxLocalFiniteElement.localBasis().evaluateJacobian(quadPos, fluxReferenceJacobians);
 
-    // Compute the flux shape function divergence on the real element
+    // Helper function to compute the trace of a matrix
+    auto trace = [](const auto& matrix) {
+      double r=0;
+      for (size_t j=0; j<matrix.N(); j++)
+        r += matrix[j][j];
+      return r;
+    };
+
+    // Domain transformation of Jacobians and computation of div = trace(Jacobian)
     std::vector<double> fluxDivergence(fluxValues.size(), 0.0);
-    for (size_t i=0; i<fluxValues.size(); i++)
-    {
-      for (size_t j=0; j<dim; j++)
-        fluxDivergence[i] += fluxReferenceGradients[i][j][j];
-      fluxDivergence[i] *= invIntegrationElement;
-    }
+    for (size_t i=0; i<fluxReferenceJacobians.size(); i++)
+      fluxDivergence[i] = trace(fluxReferenceJacobians[i] * transpose(jacInvTrans));
 
     ///////////////////////////////////////////////////////////////////////////
     // Shape functions - pressure
@@ -392,7 +388,7 @@ int main (int argc, char *argv[])
   auto topFluxBC   = [&pi] (const Domain& x) { return -0.05 * (1. - x[0]) * std::sin(2.*pi*x[0]); };
   auto lowerFluxBC = [&pi] (const Domain& x) { return  0.05 * (1. - x[0]) * std::sin(2.*pi*x[0]); };
 
-  using namespace TypeTree::Indices;
+  using namespace Dune::Indices;
   using BitVectorType = BlockVector<BlockVector<FieldVector<char,1> > >;
 
   BitVectorType isTopBoundary;
