@@ -5,8 +5,8 @@
 #include <vector>
 #include <cmath>
 
-#include <dune/common/function.hh>
 #include <dune/common/bitsetvector.hh>
+#include <dune/common/transpose.hh>
 
 #include <dune/geometry/quadraturerules.hh>
 
@@ -57,24 +57,24 @@ void getLocalMatrix( const LocalView& localView, MatrixType& elementMatrix)
     const FieldVector<double,dim>& quadPos = quad[pt].position();
 
     // The transposed inverse Jacobian of the map from the reference element to the element
-    const auto& jacobian = geometry.jacobianInverseTransposed(quadPos);
+    const auto& jacobianInverseTransposed = geometry.jacobianInverseTransposed(quadPos);
 
     // The multiplicative factor in the integral transformation formula
     const double integrationElement = geometry.integrationElement(quadPos);
 
     // The gradients of the shape functions on the reference element
-    std::vector<FieldMatrix<double,1,dim> > referenceGradients;
-    localFiniteElement.localBasis().evaluateJacobian(quadPos, referenceGradients);
+    std::vector<FieldMatrix<double,1,dim> > referenceJacobians;
+    localFiniteElement.localBasis().evaluateJacobian(quadPos, referenceJacobians);
 
     // Compute the shape function gradients on the real element
-    std::vector<FieldVector<double,dim> > gradients(referenceGradients.size());
-    for (size_t i=0; i<gradients.size(); i++)
-      jacobian.mv(referenceGradients[i][0], gradients[i]);
+    std::vector<FieldMatrix<double,1,dim> > jacobians(referenceJacobians.size());
+    for (size_t i=0; i<jacobians.size(); i++)
+      jacobians[i] = referenceJacobians[i] * transpose(jacobianInverseTransposed);
 
     // Compute the actual matrix entries
     for (size_t i=0; i<elementMatrix.N(); i++)
       for (size_t j=0; j<elementMatrix.M(); j++ )
-        elementMatrix[i][j] += ( gradients[i] * gradients[j] ) * quad[pt].weight() * integrationElement;
+        elementMatrix[i][j] += (jacobians[i] * transpose(jacobians[j])) * quad[pt].weight() * integrationElement;
 
   }
 
